/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Save File to.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


var savefileto_util =
{
    getBrowserVersion: function()
    {
        return Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo).version;
    },
    
    getExtensionVersion: function(id)
    {
        var em = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
        return em.getItemForID(id).version;
    },
    
    compareVersion: function(a, b)
    {
        return Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator).compare(a, b);
    },
    
    getPrefBranch: function(name)
    {
        return Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch(name);
    },
    
    getUnicharPref: function(prefs, name)
    {
        return prefs.getComplexValue(name, Components.interfaces.nsISupportsString).data;
    },
    
    setUnicharPref: function(prefs, name, value)
    {
        var str = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
        str.data = value;
        prefs.setComplexValue(name, Components.interfaces.nsISupportsString, str);
    },
    
    isInstantApply: function()
    {
        return savefileto_util.getPrefBranch("").getBoolPref("browser.preferences.instantApply");
    },
    
    parseFolders: function(str, withFile)
    {
        var folders = [];
        if (str)
        {
            // <name1>*<path1>|<name2>*<path2>|...
            var entries = str.split("|");
            for (var i = 0; i < entries.length; i++)
            {
                try
                {
                    var values = entries[i].split("*");
                    var folder = {name: values[0], path: values[1]};
                    if (withFile)
                        folder.file = savefileto_util.getFile(values[1]);
                    folders.push(folder);
                }
                catch (e)
                {
                    Components.utils.reportError(e);
                }
            }
        }
        return folders;
    },
    
    foldersToString: function(folders)
    {
        var array = [];
        for (var i = 0; i < folders.length; i++)
            array.push(folders[i].name + "*" + folders[i].path);
        return array.join("|");
    },
    
    getSpecialFile: function(name)
    {
        return Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get(name, Components.interfaces.nsIFile);
    },
    
    getFile: function(path)
    {
        var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
        file.initWithPath(path);
        return file;
    },
    
    appendMenuItem: function(menu, label, doOnCommand)
    {
        var menuItem = document.createElement("menuitem");
        menuItem.setAttribute("label", label);
        menuItem.addEventListener("command", doOnCommand, false);
        menu.appendChild(menuItem);
        return menuItem;
    },
    
    appendMenuSeparator: function(menu)
    {
        var menuSeparator = document.createElement("menuseparator");
        menu.appendChild(menuSeparator);
        return menuSeparator;
    },
    
    /**
     * Creates a callback function for event handling.
     * For example, if object is given and includeEvent is true,
     * obj.func(event, args...) would be called when callback is invoked.
     *
     * obj          - object or null
     * func         - function to called: name of the method
     *                or if the object is null function itself
     * includeEvent - optional; if true, the first argument for the callback is
     *                the event object;
     *                if false, the event is omitted from the argument list
     * args         - optional; other arguments to be passed on to event handler
     */
    getEventHandler: function(obj, func, includeEvent/*=true, args...*/)
    {
        if (includeEvent === undefined)
            includeEvent = true;
        var args = [];
        for (var i = 3; i < arguments.length; i++)
            args.push(arguments[i]);
        return function(e)
        {
            var _args = args;
            if (includeEvent)
            {
                _args = _args.slice(0);
                _args.unshift(e);
            }
            if (obj)
                obj[func].apply(obj, _args);
            else
                func.apply(this, _args);
        }
    },
    
    getEventStopHandler: function(obj, func, includeEvent/*=true, args...*/)
    {
        var callback = this.getEventHandler.apply(this, arguments);
        return function(e)
        {
            e.stopPropagation();
            callback(e);
        }
    },
    
    isArray: function(a)
    {
        return ((typeof(a) == "object") && (a.constructor == Array));
    },
    
    compare: function(a, b)
    {
        return ((a < b) ? -1 : ((a == b) ? 0 : 1));
    },
    
    commonPrefix: function(s1, s2)
    {
        var i = 0;
        var n = Math.max(s1.length, s2.length);
        while ((i < n) && (s1.charAt(i) == s2.charAt(i)))
            i++;
        return s1.substr(0, i);
    },
    
    trim: function(s)
    {
        return s.replace(/^\s+|\s+$/g, "");
    },
    
    ltrim: function(s)
    {
        return s.replace(/^\s+/, "");
    },
    
    rtrim: function(s)
    {
        return s.replace(/\s+$/, "");
    }
}

savefileto_util.isWin = (navigator.platform.toLowerCase().indexOf('win') > -1);
savefileto_util.pathSep = (savefileto_util.isWin ? '\\' : '/');

//function dumpln(str) { dump(str + "\n"); }
